@@ -10,7 +10,7 @@ use futures_retry::{RetryPolicy, FutureRetry, StreamRetryExt};
 pub trait EventEmitter {
     type Event;
     type Error: std::fmt::Debug;
-    async fn emit_event(&mut self, completed_events: Self::Event) -> Result<(), Self::Error>;
+    async fn emit_event(&mut self, completed_events: Vec<Self::Event>) -> Result<(), Self::Error>;
 }
 
 #[derive(Clone)]
@@ -47,20 +47,22 @@ impl<S, F> EventEmitter for S3EventEmitter<S, F>
     type Event = Vec<u8>;
     type Error = Box<dyn Error>;
 
-    async fn emit_event(&mut self, event: Self::Event) -> Result<(), Self::Error> {
-        let key = (self.key_fn)(&event);
+    async fn emit_event(&mut self, events: Vec<Self::Event>) -> Result<(), Self::Error> {
+        for event in events {
+            let key = (self.key_fn)(&event);
 
-        self.s3.put_object(
-            PutObjectRequest {
-                body: Some(event.into()),
-                bucket: self.output_bucket.clone(),
-                key,
-                ..Default::default()
-            }
-        )
-            .with_timeout(Duration::from_secs(2))
-            .compat()
-            .await?;
+            self.s3.put_object(
+                PutObjectRequest {
+                    body: Some(event.into()),
+                    bucket: self.output_bucket.clone(),
+                    key,
+                    ..Default::default()
+                }
+            )
+                .with_timeout(Duration::from_secs(2))
+                .compat()
+                .await?;
+        }
 
         Ok(())
     }
